@@ -13,9 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from tests.base import (
-    ZuulTestCase,
-)
+from tests.base import ZuulTestCase, skipIfMultiScheduler
 
 
 class TestGerritToGithubCRD(ZuulTestCase):
@@ -56,7 +54,7 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.assertFalse(B.is_merged)
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         self.executor_server.hold_jobs_in_build = True
         B.addLabel('approved')
@@ -347,6 +345,7 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
 
         # Make sure the items still share a change queue, and the
         # first one is not live.
@@ -374,9 +373,11 @@ class TestGerritToGithubCRD(ZuulTestCase):
                          (B.head_sha,))
         self.assertEqual(len(tenant.layout.pipelines['check'].queues), 0)
 
+    @skipIfMultiScheduler()
     def test_crd_check_reconfiguration(self):
         self._test_crd_check_reconfiguration('org/project1', 'org/project2')
 
+    @skipIfMultiScheduler()
     def test_crd_undefined_project(self):
         """Test that undefined projects in dependencies are handled for
         independent pipelines"""
@@ -472,8 +473,8 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.assertEqual(B.reported, 1)
 
         # Update A to add A->B (a cycle).
-        A.editBody('Depends-On: %s\n' % (B.data['url']))
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(
+            A.editBody('Depends-On: %s\n' % (B.data['url'])))
         self.waitUntilSettled()
 
         # Dependency cycle injected so zuul should have reported again on A
@@ -518,7 +519,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
         self.assertEqual(B.data['status'], 'NEW')
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         self.executor_server.hold_jobs_in_build = True
         B.addApproval('Approved', 1)
@@ -668,7 +669,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
         # should not be processed in dependent pipeline
         self.assertFalse(A.is_merged)
         self.assertEqual(B.data['status'], 'NEW')
-        self.assertEqual(len(A.comments), 0)
+        self.assertEqual(len(A.comments), 1)
         self.assertEqual(B.reported, 0)
         self.assertEqual(len(self.history), 0)
 
@@ -687,7 +688,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.assertTrue(A.is_merged)
-        self.assertEqual(len(A.comments), 2)
+        self.assertEqual(len(A.comments), 3)
         self.assertEqual(B.data['status'], 'MERGED')
         self.assertEqual(B.reported, 0)
 
@@ -701,9 +702,8 @@ class TestGithubToGerritCRD(ZuulTestCase):
             'gerrit/project1', 'master', 'B')
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % (B.data['url'],))
-
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(
+            A.editBody('Depends-On: %s\n' % (B.data['url'],)))
         self.waitUntilSettled()
 
         self.hold_jobs_in_queue = False
@@ -741,12 +741,12 @@ class TestGithubToGerritCRD(ZuulTestCase):
             'gerrit/project1', 'master', 'B')
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % (B.data['url'],))
+        event = A.editBody('Depends-On: %s\n' % (B.data['url'],))
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
         check_pipeline = tenant.layout.pipelines['check']
 
         # Add two dependent changes...
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(event)
         self.waitUntilSettled()
         self.assertEqual(len(check_pipeline.getAllItems()), 2)
 
@@ -794,12 +794,12 @@ class TestGithubToGerritCRD(ZuulTestCase):
             'gerrit/project1', 'master', 'B')
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % (B.data['url'],))
-
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(
+            A.editBody('Depends-On: %s\n' % (B.data['url'],)))
         self.waitUntilSettled()
 
         self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
 
         # Make sure the items still share a change queue, and the
         # first one is not live.
@@ -827,9 +827,11 @@ class TestGithubToGerritCRD(ZuulTestCase):
                          (A.head_sha,))
         self.assertEqual(len(tenant.layout.pipelines['check'].queues), 0)
 
+    @skipIfMultiScheduler()
     def test_crd_check_reconfiguration(self):
         self._test_crd_check_reconfiguration('org/project1', 'org/project2')
 
+    @skipIfMultiScheduler()
     def test_crd_undefined_project(self):
         """Test that undefined projects in dependencies are handled for
         independent pipelines"""
@@ -856,9 +858,8 @@ class TestGithubToGerritCRD(ZuulTestCase):
             B.subject, C.url)
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % (B.data['url'],))
-
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(
+            A.editBody('Depends-On: %s\n' % (B.data['url'],)))
         self.waitUntilSettled()
         self.assertEqual(self.history[-1].changes, '2,%s 1,1 1,%s' %
                          (C.head_sha, A.head_sha))
@@ -900,14 +901,14 @@ class TestGithubToGerritCRD(ZuulTestCase):
             'gerrit/unknown', 'master', 'B')
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % (B.data['url'],))
+        event = A.editBody('Depends-On: %s\n' % (B.data['url'],))
 
         # Make sure zuul has seen an event on B.
         self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
         # Note we wait until settled here as the event processing for
         # the next event may not have the updated db yet otherwise.
         self.waitUntilSettled()
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(event)
         self.waitUntilSettled()
 
         self.assertFalse(A.is_merged)
@@ -927,8 +928,8 @@ class TestGithubToGerritCRD(ZuulTestCase):
         # Create B->A
         B = self.fake_github.openFakePullRequest('github/project2', 'master',
                                                  'B')
-        B.editBody('Depends-On: %s\n' % (A.data['url'],))
-        self.fake_github.emitEvent(B.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(
+            B.editBody('Depends-On: %s\n' % (A.data['url'],)))
         self.waitUntilSettled()
 
         # Dep is there so zuul should have reported on B
