@@ -16,6 +16,7 @@
 from tests.base import (
     ZuulTestCase,
     simple_layout,
+    skipIfMultiScheduler,
 )
 
 URL_FORMATS = [
@@ -73,7 +74,7 @@ class TestGerritCRD(ZuulTestCase):
         self.assertEqual(B.data['status'], 'NEW')
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         self.executor_server.hold_jobs_in_build = True
         B.addApproval('Approved', 1)
@@ -252,7 +253,7 @@ class TestGerritCRD(ZuulTestCase):
 
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
-        self.assertEqual(A.reported, 0)
+        self.assertEqual(A.reported, 1)
         self.assertEqual(B.reported, 0)
         self.assertEqual(len(self.history), 0)
 
@@ -269,7 +270,7 @@ class TestGerritCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.assertEqual(A.data['status'], 'MERGED')
-        self.assertEqual(A.reported, 2)
+        self.assertEqual(A.reported, 3)
 
     def _test_crd_gate_reverse(self, url_fmt):
         "Test reverse cross-repo dependencies"
@@ -368,7 +369,7 @@ class TestGerritCRD(ZuulTestCase):
         # should not be processed in dependent pipeline
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
-        self.assertEqual(A.reported, 0)
+        self.assertEqual(A.reported, 1)
         self.assertEqual(B.reported, 0)
         self.assertEqual(len(self.history), 0)
 
@@ -387,7 +388,7 @@ class TestGerritCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.assertEqual(A.data['status'], 'MERGED')
-        self.assertEqual(A.reported, 2)
+        self.assertEqual(A.reported, 3)
         self.assertEqual(B.data['status'], 'MERGED')
         self.assertEqual(B.reported, 0)
 
@@ -430,7 +431,6 @@ class TestGerritCRD(ZuulTestCase):
 
     def test_crd_check_git_depends(self):
         "Test single-repo dependencies in independent pipelines"
-        self.gearman_server.hold_jobs_in_build = True
         A = self.fake_gerrit.addFakeChange('org/project1', 'master', 'A')
         B = self.fake_gerrit.addFakeChange('org/project1', 'master', 'B')
 
@@ -442,10 +442,6 @@ class TestGerritCRD(ZuulTestCase):
         self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
         self.waitUntilSettled()
 
-        self.orderedRelease()
-        self.gearman_server.hold_jobs_in_build = False
-        self.waitUntilSettled()
-
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
         self.assertEqual(A.reported, 1)
@@ -517,6 +513,7 @@ class TestGerritCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
 
         # Make sure the items still share a change queue, and the
         # first one is not live.
@@ -541,9 +538,11 @@ class TestGerritCRD(ZuulTestCase):
         self.assertEqual(self.history[0].changes, '2,1 1,1')
         self.assertEqual(len(tenant.layout.pipelines['check'].queues), 0)
 
+    @skipIfMultiScheduler()
     def test_crd_check_reconfiguration(self):
         self._test_crd_check_reconfiguration('org/project1', 'org/project2')
 
+    @skipIfMultiScheduler()
     def test_crd_undefined_project(self):
         """Test that undefined projects in dependencies are handled for
         independent pipelines"""
