@@ -201,6 +201,54 @@ class TestInventoryAutoPython(TestInventoryBase):
         self.executor_server.release()
         self.waitUntilSettled()
 
+    def test_auto_python_ansible29_inventory(self):
+        inventory = self._get_build_inventory('ansible-version29-inventory')
+
+        all_nodes = ('ubuntu-xenial',)
+        self.assertIn('all', inventory)
+        self.assertIn('hosts', inventory['all'])
+        self.assertIn('vars', inventory['all'])
+        for node_name in all_nodes:
+            self.assertIn(node_name, inventory['all']['hosts'])
+            node_vars = inventory['all']['hosts'][node_name]
+            self.assertEqual(
+                'auto', node_vars['ansible_python_interpreter'])
+
+        self.assertIn('zuul', inventory['all']['vars'])
+        z_vars = inventory['all']['vars']['zuul']
+        self.assertIn('executor', z_vars)
+        self.assertIn('src_root', z_vars['executor'])
+        self.assertIn('job', z_vars)
+        self.assertEqual(z_vars['job'], 'ansible-version29-inventory')
+        self.assertEqual(z_vars['message'], 'QQ==')
+
+        self.executor_server.release()
+        self.waitUntilSettled()
+
+    def test_auto_python_ansible5_inventory(self):
+        inventory = self._get_build_inventory('ansible-version5-inventory')
+
+        all_nodes = ('ubuntu-xenial',)
+        self.assertIn('all', inventory)
+        self.assertIn('hosts', inventory['all'])
+        self.assertIn('vars', inventory['all'])
+        for node_name in all_nodes:
+            self.assertIn(node_name, inventory['all']['hosts'])
+            node_vars = inventory['all']['hosts'][node_name]
+            self.assertEqual(
+                'auto', node_vars['ansible_python_interpreter'])
+
+        self.assertIn('zuul', inventory['all']['vars'])
+        z_vars = inventory['all']['vars']['zuul']
+        self.assertIn('executor', z_vars)
+        self.assertIn('src_root', z_vars['executor'])
+        self.assertIn('job', z_vars)
+        self.assertEqual(z_vars['job'], 'ansible-version5-inventory')
+        self.assertEqual(z_vars['message'], 'QQ==')
+
+        self.executor_server.release()
+        self.waitUntilSettled()
+
 
 class TestInventory(TestInventoryBase):
 
@@ -397,7 +445,8 @@ class TestAnsibleInventory(AnsibleZuulTestCase):
         # Output extra ansible info so we might see errors.
         self.executor_server.verbose = True
         A = self.fake_gerrit.addFakeChange(
-            'org/project2', 'master', expected_message)
+            'org/project2', 'master', expected_message,
+            files={'jinja.txt': 'foo'})
         self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
         self.waitUntilSettled()
         self.assertHistory([
@@ -429,6 +478,19 @@ class TestAnsibleInventory(AnsibleZuulTestCase):
     def test_jinja2_message_raw(self):
         self._jinja2_message("This message has {% raw %} in {% endraw %} it ")
 
+    def test_network_inventory(self):
+        # Network appliances can't run the freeze or setup playbooks,
+        # so they won't have any job variables available.  But they
+        # should still have nodepool hostvars.  Run a playbook that
+        # verifies that.
+        A = self.fake_gerrit.addFakeChange(
+            'org/project2', 'master', 'A',
+            files={'network.txt': 'foo'})
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        self.assertHistory([
+            dict(name='network', result='SUCCESS', changes='1,1')])
+
 
 class TestWindowsInventory(TestInventoryBase):
     config_file = 'zuul-winrm.conf'
