@@ -184,12 +184,14 @@ class TestMergerRepo(ZuulTestCase):
         work_repo = Repo(parent_path, self.workspace_root,
                          'none@example.org', 'User Name', '0', '0')
         work_repo.setRemoteRef('master', commit_sha)
-        work_repo.setRemoteRef('invalid', commit_sha)
+        # missing remote ref would be created
+        work_repo.setRemoteRef('missing', commit_sha)
 
         repo = git.Repo(self.workspace_root)
         self.assertEqual(repo.remotes.origin.refs.master.commit.hexsha,
                          commit_sha)
-        self.assertNotIn('invalid', repo.remotes.origin.refs)
+        self.assertEqual(repo.remotes.origin.refs.missing.commit.hexsha,
+                         commit_sha)
 
     def test_clone_timeout(self):
         parent_path = os.path.join(self.upstream_root, 'org/project1')
@@ -363,6 +365,39 @@ class TestMergerRepo(ZuulTestCase):
         self.assertEqual(sorted(['README', 'feature.txt']),
                          sorted(changed_files))
 
+    def test_files_changes_add_and_remove_files(self):
+        """
+        If the changed files in previous commits are reverted in later commits,
+        they should not be considered as changed in the PR.
+        """
+        parent_path = os.path.join(self.upstream_root, 'org/project1')
+        self.create_branch('org/project1', 'feature1')
+
+        base_sha = git.Repo(parent_path).commit('master').hexsha
+
+        # Let the file that is also changed in the feature branch diverge
+        # in master. This change should NOT be considered in the changed
+        # files list.
+        files = {'to-be-deleted.txt': 'FAIL'}
+        self.create_commit('org/project1', files=files, head='master',
+                           message='Add master file')
+        work_repo = Repo(parent_path, self.workspace_root,
+                         'none@example.org', 'User Name', '0', '0')
+        # Add a file in first commit
+        files = {'to-be-deleted.txt': 'test'}
+        self.create_commit('org/project1', files=files, head='feature1',
+                           message='Add file')
+        changed_files = work_repo.getFilesChanges('feature1', base_sha)
+        self.assertEqual(sorted(['README', 'to-be-deleted.txt']),
+                         sorted(changed_files))
+        # Delete the file in second commit
+        delete_files = ['to-be-deleted.txt']
+        self.create_commit('org/project1', files={},
+                           delete_files=delete_files, head='feature1',
+                           message='Delete file')
+        changed_files = work_repo.getFilesChanges('feature1', base_sha)
+        self.assertEqual(['README'], changed_files)
+
     def test_files_changes_master_fork_merges(self):
         """Regression test for getFilesChanges()
 
@@ -1133,7 +1168,13 @@ class TestMergerSchemes(ZuulTestCase):
                 os.path.join(root, 'project1')))
 
     def test_unique_scheme(self):
-        merger = self._getMerger(scheme=zuul.model.SCHEME_UNIQUE)
+        cache_merger = self._getMerger(work_root=self.cache_root)
+        cache_merger.updateRepo('gerrit', 'org/project1')
+        self._assertScheme(self.cache_root, 'unique')
+
+        merger = self._getMerger(
+            cache_root=self.cache_root,
+            scheme=zuul.model.SCHEME_UNIQUE)
         merger.getRepo('gerrit', 'org/project1')
         self._assertScheme(self.work_root, 'unique')
 
