@@ -18,7 +18,6 @@ import PropTypes from 'prop-types'
 import { PageSection, PageSectionVariants } from '@patternfly/react-core'
 
 import Job from '../containers/job/Job'
-import { Fetchable } from '../containers/Fetching'
 import { fetchJobIfNeeded } from '../actions/job'
 
 
@@ -55,12 +54,6 @@ class JobPage extends React.Component {
     const jobName = this.props.match.params.jobName
     return (
       <PageSection variant={PageSectionVariants.light}>
-        <PageSection style={{paddingRight: '5px'}}>
-          <Fetchable
-            isFetching={remoteData.isFetching}
-            fetchCallback={this.updateData}
-          />
-        </PageSection>
         {tenantJobs && tenantJobs[jobName] && <Job job={tenantJobs[jobName]} />}
       </PageSection>
     )
