@@ -16,10 +16,16 @@ export default {
     isFetchingOutput: false,
     isFetchingManifest: false,
   },
+  component: {
+    components: undefined,
+    isFetching: false,
+  },
   logfile: {
     // Store files by buildId->filename->content
     files: {},
     isFetching: false,
     url: null,
   },
+  auth: {},
+  user: {},
 }
