@@ -16,19 +16,28 @@
 import argparse
 import babel.dates
 import datetime
+import json
 import jwt
 import logging
 import prettytable
+import os
 import re
 import sys
 import time
 import textwrap
 import requests
 import urllib.parse
+from uuid import uuid4
 
-import zuul.rpcclient
 import zuul.cmd
 from zuul.lib.config import get_default
+from zuul.model import SystemAttributes, PipelineState
+from zuul.zk import ZooKeeperClient
+from zuul.lib.keystorage import KeyStorage
+from zuul.zk.locks import tenant_write_lock
+from zuul.zk.zkobject import ZKContext
+from zuul.zk.layout import LayoutState, LayoutStateStore
+from zuul.zk.components import COMPONENT_REGISTRY
 
 
 # todo This should probably live somewhere else
@@ -178,6 +187,7 @@ class Client(zuul.cmd.ZuulApp):
                                            description='valid commands',
                                            help='additional help')
 
+        # Autohold
         cmd_autohold = subparsers.add_parser(
             'autohold', help='hold nodes for failed job')
         cmd_autohold.add_argument('--tenant', help='tenant name',
@@ -221,6 +231,7 @@ class Client(zuul.cmd.ZuulApp):
                                        required=True)
         cmd_autohold_list.set_defaults(func=self.autohold_list)
 
+        # Enqueue/Dequeue
         cmd_enqueue = subparsers.add_parser('enqueue', help='enqueue a change')
         cmd_enqueue.add_argument('--tenant', help='tenant name',
                                  required=True)
@@ -277,6 +288,7 @@ class Client(zuul.cmd.ZuulApp):
                                  default=None)
         cmd_dequeue.set_defaults(func=self.dequeue)
 
+        # Promote
         cmd_promote = subparsers.add_parser('promote',
                                             help='promote one or more changes')
         cmd_promote.add_argument('--tenant', help='tenant name',
@@ -287,6 +299,7 @@ class Client(zuul.cmd.ZuulApp):
                                  required=True, nargs='+')
         cmd_promote.set_defaults(func=self.promote)
 
+        # Show
         cmd_show = subparsers.add_parser('show',
                                          help='show current statuses')
         cmd_show.set_defaults(func=self.show_running_jobs)
@@ -306,11 +319,13 @@ class Client(zuul.cmd.ZuulApp):
         # TODO: add filters such as queue, project, changeid etc
         show_running_jobs.set_defaults(func=self.show_running_jobs)
 
+        # Conf check
         cmd_conf_check = subparsers.add_parser(
             'tenant-conf-check',
             help='validate the tenant configuration')
         cmd_conf_check.set_defaults(func=self.validate)
 
+        # Auth token
         cmd_create_auth_token = subparsers.add_parser(
             'create-auth-token',
             help='create an Authentication Token for the web API',
@@ -349,6 +364,127 @@ class Client(zuul.cmd.ZuulApp):
             required=False)
         cmd_create_auth_token.set_defaults(func=self.create_auth_token)
 
+        # Key storage
+        cmd_import_keys = subparsers.add_parser(
+            'import-keys',
+            help='import project keys to ZooKeeper',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Import previously exported project secret keys to ZooKeeper
+
+            Given a file with previously exported project keys, this
+            command will import them into ZooKeeper.  Existing keys
+            will not be overwritten; to overwrite keys, add the
+            --force flag.'''))
+        cmd_import_keys.set_defaults(command='import-keys')
+        cmd_import_keys.add_argument('path', type=str,
+                                     help='key export file path')
+        cmd_import_keys.add_argument('--force', action='store_true',
+                                     help='overwrite existing keys')
+        cmd_import_keys.set_defaults(func=self.import_keys)
+
+        cmd_export_keys = subparsers.add_parser(
+            'export-keys',
+            help='export project keys from ZooKeeper',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Export project secret keys from ZooKeeper
+
+            This command exports project secret keys from ZooKeeper
+            and writes them to a file which is suitable for backing
+            up and later use with the import-keys command.
+
+            The key contents are still protected by the keystore
+            password and can not be used or decrypted without it.'''))
+        cmd_export_keys.set_defaults(command='export-keys')
+        cmd_export_keys.add_argument('path', type=str,
+                                     help='key export file path')
+        cmd_export_keys.set_defaults(func=self.export_keys)
+
+        cmd_copy_keys = subparsers.add_parser(
+            'copy-keys',
+            help='copy keys from one project to another',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Copy secret keys from one project to another
+
+            When projects are renamed, this command may be used to
+            copy the secret keys from the current name to the new name
+            in order to avoid service interruption.'''))
+        cmd_copy_keys.set_defaults(command='copy-keys')
+        cmd_copy_keys.add_argument('src_connection', type=str,
+                                   help='original connection name')
+        cmd_copy_keys.add_argument('src_project', type=str,
+                                   help='original project name')
+        cmd_copy_keys.add_argument('dest_connection', type=str,
+                                   help='new connection name')
+        cmd_copy_keys.add_argument('dest_project', type=str,
+                                   help='new project name')
+        cmd_copy_keys.set_defaults(func=self.copy_keys)
+
+        cmd_delete_keys = subparsers.add_parser(
+            'delete-keys',
+            help='delete project keys',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Delete the ssh and secrets keys for a project
+            '''))
+        cmd_delete_keys.set_defaults(command='delete-keys')
+        cmd_delete_keys.add_argument('connection', type=str,
+                                     help='connection name')
+        cmd_delete_keys.add_argument('project', type=str,
+                                     help='project name')
+        cmd_delete_keys.set_defaults(func=self.delete_keys)
+
+        # ZK Maintenance
+        cmd_delete_state = subparsers.add_parser(
+            'delete-state',
+            help='delete ephemeral ZooKeeper state',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Delete all ephemeral state stored in ZooKeeper
+
+            Zuul stores a considerable amount of ephemeral state
+            information in ZooKeeper.  Generally it should be able to
+            detect and correct any errors, but if the state becomes
+            corrupted and it is unable to recover, this command may be
+            used to delete all ephemeral data from ZooKeeper and start
+            anew.
+
+            Do not run this command while any Zuul component is
+            running (perform a complete shutdown first).
+
+            This command will only remove ephemeral Zuul data from
+            ZooKeeper; it will not remove private keys or Nodepool
+            data.'''))
+        cmd_delete_state.set_defaults(command='delete-state')
+        cmd_delete_state.set_defaults(func=self.delete_state)
+
+        cmd_delete_pipeline_state = subparsers.add_parser(
+            'delete-pipeline-state',
+            help='delete single pipeline ZooKeeper state',
+            formatter_class=argparse.RawDescriptionHelpFormatter,
+            description=textwrap.dedent('''\
+            Delete a single pipeline state stored in ZooKeeper
+
+            In the unlikely event that a bug in Zuul or ZooKeeper data
+            corruption occurs in such a way that it only affects a
+            single pipeline, this command might be useful in
+            attempting to recover.
+
+            The circumstances under which this command will be able to
+            effect a recovery are very rare and even so it may not be
+            sufficient.  In general, if an error occurs it is better
+            to shut Zuul down and run "zuul delete-state".
+
+            This command will lock the specified tenant and
+            then completely delete the pipeline state.'''))
+        cmd_delete_pipeline_state.set_defaults(command='delete-pipeline-state')
+        cmd_delete_pipeline_state.set_defaults(func=self.delete_pipeline_state)
+        cmd_delete_pipeline_state.add_argument('tenant', type=str,
+                                               help='tenant name')
+        cmd_delete_pipeline_state.add_argument('pipeline', type=str,
+                                               help='pipeline name')
         return parser
 
     def parseArguments(self, args=None):
@@ -399,18 +535,7 @@ class Client(zuul.cmd.ZuulApp):
                                     self.args.auth_token)
             return client
         conf_sections = self.config.sections()
-        if 'gearman' in conf_sections:
-            self.log.debug('gearman section found in config, using RPC client')
-            server = self.config.get('gearman', 'server')
-            port = get_default(self.config, 'gearman', 'port', 4730)
-            ssl_key = get_default(self.config, 'gearman', 'ssl_key')
-            ssl_cert = get_default(self.config, 'gearman', 'ssl_cert')
-            ssl_ca = get_default(self.config, 'gearman', 'ssl_ca')
-            client = zuul.rpcclient.RPCClient(
-                server, port, ssl_key,
-                ssl_cert, ssl_ca,
-                client_id=self.app_description)
-        elif 'webclient' in conf_sections:
+        if 'webclient' in conf_sections:
             self.log.debug('web section found in config, using REST client')
             server = get_default(self.config, 'webclient', 'url', None)
             verify = get_default(self.config, 'webclient', 'verify_ssl',
@@ -418,8 +543,8 @@ class Client(zuul.cmd.ZuulApp):
             client = ZuulRESTClient(server, verify,
                                     self.args.auth_token)
         else:
-            print('Unable to find a way to connect to Zuul, add a "gearman" '
-                  'or "web" section to your configuration file')
+            print('Unable to find a way to connect to Zuul, add a '
+                  '"web" section to your configuration file')
             sys.exit(1)
         if server is None:
             print('Missing "server" configuration value')
@@ -724,12 +849,14 @@ class Client(zuul.cmd.ZuulApp):
                 self.connections = connections
                 self.unparsed_config_cache = None
 
-        sched = SchedulerConfig(self.config, self.connections)
+        zuul_globals = SystemAttributes.fromConfig(self.config)
         loader = configloader.ConfigLoader(
-            sched.connections, sched, None, None)
+            self.connections, None, zuul_globals)
+        sched = SchedulerConfig(self.config, self.connections)
         tenant_config, script = sched._checkTenantSourceConf(self.config)
-        unparsed_abide = loader.readConfig(tenant_config, from_script=script)
         try:
+            unparsed_abide = loader.readConfig(
+                tenant_config, from_script=script)
             for conf_tenant in unparsed_abide.tenants.values():
                 loader.tenant_parser.getSchema()(conf_tenant)
             print("Tenants config validated with success")
@@ -741,6 +868,126 @@ class Client(zuul.cmd.ZuulApp):
         finally:
             sys.exit(err_code)
 
+    def export_keys(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+        try:
+            password = self.config["keystore"]["password"]
+        except KeyError:
+            raise RuntimeError("No key store password configured!")
+        keystore = KeyStorage(zk_client, password=password)
+        export = keystore.exportKeys()
+        with open(os.open(self.args.path,
+                          os.O_CREAT | os.O_WRONLY, 0o600), 'w') as f:
+            json.dump(export, f)
+        sys.exit(0)
+
+    def import_keys(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+        try:
+            password = self.config["keystore"]["password"]
+        except KeyError:
+            raise RuntimeError("No key store password configured!")
+        keystore = KeyStorage(zk_client, password=password)
+        with open(self.args.path, 'r') as f:
+            import_data = json.load(f)
+        keystore.importKeys(import_data, self.args.force)
+        sys.exit(0)
+
+    def copy_keys(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+        try:
+            password = self.config["keystore"]["password"]
+        except KeyError:
+            raise RuntimeError("No key store password configured!")
+        keystore = KeyStorage(zk_client, password=password)
+        args = self.args
+        # Load
+        ssh = keystore.loadProjectSSHKeys(args.src_connection,
+                                          args.src_project)
+        secrets = keystore.loadProjectsSecretsKeys(args.src_connection,
+                                                   args.src_project)
+        # Save
+        keystore.saveProjectSSHKeys(args.dest_connection,
+                                    args.dest_project, ssh)
+        keystore.saveProjectsSecretsKeys(args.dest_connection,
+                                         args.dest_project, secrets)
+        self.log.info("Copied keys from %s %s to %s %s",
+                      args.src_connection, args.src_project,
+                      args.dest_connection, args.dest_project)
+        sys.exit(0)
+
+    def delete_keys(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+        try:
+            password = self.config["keystore"]["password"]
+        except KeyError:
+            raise RuntimeError("No key store password configured!")
+        keystore = KeyStorage(zk_client, password=password)
+        args = self.args
+        keystore.deleteProjectSSHKeys(args.connection, args.project)
+        keystore.deleteProjectsSecretsKeys(args.connection, args.project)
+        keystore.deleteProjectDir(args.connection, args.project)
+        self.log.info("Delete keys from %s %s",
+                      args.connection, args.project)
+        sys.exit(0)
+
+    def delete_state(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+        confirm = input("Are you sure you want to delete "
+                        "all ephemeral data from ZooKeeper? (yes/no) ")
+        if confirm.strip().lower() == 'yes':
+            zk_client.client.delete('/zuul', recursive=True)
+        sys.exit(0)
+
+    def delete_pipeline_state(self):
+        logging.basicConfig(level=logging.INFO)
+
+        zk_client = ZooKeeperClient.fromConfig(self.config)
+        zk_client.connect()
+
+        args = self.args
+        safe_tenant = urllib.parse.quote_plus(args.tenant)
+        safe_pipeline = urllib.parse.quote_plus(args.pipeline)
+        COMPONENT_REGISTRY.create(zk_client)
+        with tenant_write_lock(zk_client, args.tenant) as lock:
+            path = f'/zuul/tenant/{safe_tenant}/pipeline/{safe_pipeline}'
+            layout_uuid = None
+            zk_client.client.delete(
+                f'/zuul/tenant/{safe_tenant}/pipeline/{safe_pipeline}',
+                recursive=True)
+            context = ZKContext(zk_client, lock, None, self.log)
+            ps = PipelineState.new(context, _path=path,
+                                   layout_uuid=layout_uuid)
+            # Force everyone to make a new layout for this tenant in
+            # order to rebuild the shared change queues.
+            layout_state = LayoutState(
+                tenant_name=args.tenant,
+                hostname='admin command',
+                last_reconfigured=int(time.time()),
+                uuid=uuid4().hex,
+                branch_cache_min_ltimes={},
+                ltime=ps._zstat.last_modified_transaction_id,
+            )
+            tenant_layout_state = LayoutStateStore(zk_client, lambda: None)
+            tenant_layout_state[args.tenant] = layout_state
+
+        sys.exit(0)
+
 
 def main():
     Client().main()
