@@ -203,6 +203,7 @@ class GerritSource(BaseSource):
             connection_name=self.connection.connection_name,
             open=config.get('open'),
             current_patchset=config.get('current-patchset'),
+            wip=config.get('wip'),
             statuses=to_list(config.get('status')),
             required_approvals=to_list(config.get('approval')),
         )
@@ -234,6 +235,7 @@ def getRequireSchema():
     require = {'approval': scalar_or_list(approval),
                'open': bool,
                'current-patchset': bool,
+               'wip': bool,
                'status': scalar_or_list(str)}
 
     return require
