@@ -18,9 +18,9 @@ import urllib
 
 from zuul.model import Project
 from zuul.source import BaseSource
-
 from zuul.driver.gitlab.gitlabmodel import GitlabRefFilter
 from zuul.driver.util import scalar_or_list, to_list
+from zuul.zk.change_cache import ChangeKey
 
 
 class GitlabSource(BaseSource):
@@ -57,8 +57,30 @@ class GitlabSource(BaseSource):
         """Called after configuration has been processed."""
         raise NotImplementedError()
 
-    def getChange(self, event, refresh=False):
-        return self.connection.getChange(event, refresh)
+    def getChangeKey(self, event):
+        connection_name = self.connection.connection_name
+        if event.change_number:
+            return ChangeKey(connection_name, event.project_name,
+                             'MergeRequest',
+                             str(event.change_number),
+                             str(event.patch_number))
+        revision = f'{event.oldrev}..{event.newrev}'
+        if event.ref and event.ref.startswith('refs/tags/'):
+            tag = event.ref[len('refs/tags/'):]
+            return ChangeKey(connection_name, event.project_name,
+                             'Tag', tag, revision)
+        if event.ref and event.ref.startswith('refs/heads/'):
+            branch = event.ref[len('refs/heads/'):]
+            return ChangeKey(connection_name, event.project_name,
+                             'Branch', branch, revision)
+        if event.ref:
+            return ChangeKey(connection_name, event.project_name,
+                             'Ref', event.ref, revision)
+        self.log.warning("Unable to format change key for %s" % (self,))
+
+    def getChange(self, change_key, refresh=False, event=None):
+        return self.connection.getChange(change_key, refresh=refresh,
+                                         event=event)
 
     def getChangeByURL(self, url, event):
         try:
@@ -76,10 +98,10 @@ class GitlabSource(BaseSource):
         mr = self.connection.getMR(project_name, num)
         if not mr:
             return None
-        project = self.getProject(project_name)
-        change = self.connection._getChange(
-            project, num, mr['sha'], url=url,
-            event=event)
+        change_key = ChangeKey(self.connection.connection_name, project_name,
+                               'MergeRequest',
+                               str(num), mr['sha'])
+        change = self.connection._getChange(change_key, event=event)
         return change
 
     def getChangesDependingOn(self, change, projects, tenant):
@@ -87,7 +109,7 @@ class GitlabSource(BaseSource):
             change, projects, tenant)
 
     def getCachedChanges(self):
-        return list(self.connection._change_cache.values())
+        yield from self.connection._change_cache
 
     def getProject(self, name):
         p = self.connection.getProject(name)
@@ -96,8 +118,11 @@ class GitlabSource(BaseSource):
             self.connection.addProject(p)
         return p
 
-    def getProjectBranches(self, project, tenant):
-        return self.connection.getProjectBranches(project, tenant)
+    def getProjectBranches(self, project, tenant, min_ltime=-1):
+        return self.connection.getProjectBranches(project, tenant, min_ltime)
+
+    def getProjectBranchCacheLtime(self):
+        return self.connection._branch_cache.ltime
 
     def getProjectOpenChanges(self, project):
         """Get the open changes for a project."""
@@ -131,6 +156,9 @@ class GitlabSource(BaseSource):
     def getRefForChange(self, change):
         raise NotImplementedError()
 
+    def setChangeAttributes(self, change, **attrs):
+        return self.connection.updateChangeAttributes(change, **attrs)
+
 
 # Require model
 def getRequireSchema():
