@@ -23,11 +23,18 @@ import requests
 import cherrypy
 import voluptuous as v
 
-from zuul.connection import BaseConnection
+from zuul.connection import (
+    BaseConnection, ZKChangeCacheMixin, ZKBranchCacheMixin
+)
 from zuul.lib.logutil import get_annotated_logger
 from zuul.web.handler import BaseWebController
 from zuul.model import Ref, Branch, Tag
 from zuul.lib import dependson
+from zuul.zk.branch_cache import BranchCache
+from zuul.zk.change_cache import (
+    AbstractChangeCache,
+    ConcurrentUpdateError,
+)
 from zuul.zk.event_queues import ConnectionEventQueue
 
 from zuul.driver.pagure.paguremodel import PagureTriggerEvent, PullRequest
@@ -96,6 +103,17 @@ def _sign_request(body, secret):
     return signature, body
 
 
+class PagureChangeCache(AbstractChangeCache):
+    log = logging.getLogger("zuul.driver.PagureChangeCache")
+
+    CHANGE_TYPE_MAP = {
+        "Ref": Ref,
+        "Tag": Tag,
+        "Branch": Branch,
+        "PullRequest": PullRequest,
+    }
+
+
 class PagureEventConnector(threading.Thread):
     """Move events from Pagure into the scheduler"""
 
@@ -137,6 +155,11 @@ class PagureEventConnector(threading.Thread):
         return not self._stopped
 
     def run(self):
+        # Wait for the scheduler to prime its config so that we have
+        # the full tenant list before we start moving events.
+        self.connection.sched.primed_event.wait()
+        if self._stopped:
+            return
         self.event_queue.registerEventWatch(self._onNewEvent)
         while not self._stopped:
             try:
@@ -188,12 +211,8 @@ class PagureEventConnector(threading.Thread):
         if event:
             event.timestamp = timestamp
             if event.change_number:
-                project = self.connection.source.getProject(event.project_name)
-                self.connection._getChange(project,
-                                           event.change_number,
-                                           event.patch_number,
-                                           refresh=True,
-                                           url=event.change_url,
+                change_key = self.connection.source.getChangeKey(event)
+                self.connection._getChange(change_key, refresh=True,
                                            event=event)
             event.project_hostname = self.connection.canonical_hostname
             self.connection.logEvent(event)
@@ -230,6 +249,7 @@ class PagureEventConnector(threading.Thread):
         """ Handles pull request initial comment change """
         event, _ = self._event_base(body)
         event.action = 'changed'
+        event.initial_comment_changed = True
         return event
 
     def _event_pull_request_tags_changed(self, body):
@@ -314,9 +334,7 @@ class PagureEventConnector(threading.Thread):
         event.ref = 'refs/heads/%s' % event.branch
         event.newrev = data.get('rev')
         event.oldrev = '0' * 40
-        event.branch_created = True
-        self.connection.project_branch_cache[
-            event.project_name].append(event.branch)
+        self.connection.clearConnectionCacheOnBranchEvent(event)
         return event
 
     def _event_ref_deleted(self, body):
@@ -327,9 +345,7 @@ class PagureEventConnector(threading.Thread):
         event.ref = 'refs/heads/%s' % event.branch
         event.oldrev = data.get('rev')
         event.newrev = '0' * 40
-        event.branch_deleted = True
-        self.connection.project_branch_cache[
-            event.project_name].remove(event.branch)
+        self.connection.clearConnectionCacheOnBranchEvent(event)
         return event
 
 
@@ -448,15 +464,13 @@ class PagureAPIClient():
         return resp[0]['webhook']['token']
 
 
-class PagureConnection(BaseConnection):
+class PagureConnection(ZKChangeCacheMixin, ZKBranchCacheMixin, BaseConnection):
     driver_name = 'pagure'
     log = logging.getLogger("zuul.PagureConnection")
 
     def __init__(self, driver, connection_name, connection_config):
         super(PagureConnection, self).__init__(
             driver, connection_name, connection_config)
-        self._change_cache = {}
-        self.project_branch_cache = {}
         self.projects = {}
         self.server = self.connection_config.get('server', 'pagure.io')
         self.canonical_hostname = self.connection_config.get(
@@ -478,12 +492,29 @@ class PagureConnection(BaseConnection):
             r"^\*\*Metadata Update", re.MULTILINE)
         self.sched = None
 
-    def onLoad(self):
+    def onLoad(self, zk_client, component_registry):
         self.log.info('Starting Pagure connection: %s', self.connection_name)
+
+        # Set the project branch cache to read only if no scheduler is
+        # provided to prevent fetching the branches from the connection.
+        self.read_only = not self.sched
+
+        self.log.debug('Creating Zookeeper branch cache')
+        self._branch_cache = BranchCache(zk_client, self, component_registry)
+
         self.log.info('Creating Zookeeper event queue')
         self.event_queue = ConnectionEventQueue(
-            self.sched.zk_client, self.connection_name
+            zk_client, self.connection_name
         )
+
+        # If the connection was not loaded by a scheduler, but by e.g.
+        # zuul-web, we want to stop here.
+        if not self.sched:
+            return
+
+        self.log.debug('Creating Zookeeper change cache')
+        self._change_cache = PagureChangeCache(zk_client, self)
+
         self.log.info('Starting event connector')
         self._start_event_connector()
 
@@ -526,14 +557,6 @@ class PagureConnection(BaseConnection):
                 "Fetching project %s webhook token from API" % project)
             return token
 
-    def maintainCache(self, relevant):
-        remove = set()
-        for key, change in self._change_cache.items():
-            if change not in relevant:
-                remove.add(key)
-        for key in remove:
-            del self._change_cache[key]
-
     def getWebController(self, zuul_web):
         return PagureWebController(zuul_web, self)
 
@@ -558,83 +581,105 @@ class PagureConnection(BaseConnection):
             url += '/c/%s' % sha
         return url
 
-    def getProjectBranches(self, project, tenant):
-        branches = self.project_branch_cache.get(project.name)
-
-        if branches is not None:
-            return branches
-
+    def _fetchProjectBranches(self, project, exclude_unprotected):
         pagure = self.get_project_api_client(project.name)
         branches = pagure.get_project_branches()
-        self.project_branch_cache[project.name] = branches
 
         self.log.info("Got branches for %s" % project.name)
         return branches
 
+    def isBranchProtected(self, project_name, branch_name,
+                          zuul_event_id=None):
+        return True
+
     def getGitUrl(self, project):
         return '%s/%s' % (self.cloneurl, project.name)
 
-    def getChange(self, event, refresh=False):
-        project = self.source.getProject(event.project_name)
-        if event.change_number:
+    def getChange(self, change_key, refresh=False, event=None):
+        if change_key.connection_name != self.connection_name:
+            return None
+        if change_key.change_type == 'PullRequest':
             self.log.info("Getting change for %s#%s" % (
-                project, event.change_number))
-            change = self._getChange(
-                project, event.change_number, event.patch_number,
-                refresh=refresh, event=event)
-            change.is_current_patchset = (change.pr.get('commit_stop') ==
-                                          event.patch_number)
+                change_key.project_name, change_key.stable_id))
+            change = self._getChange(change_key,
+                                     refresh=refresh, event=event)
         else:
             self.log.info("Getting change for %s ref:%s" % (
-                project, event.ref))
-            if event.ref and event.ref.startswith('refs/tags/'):
-                change = Tag(project)
-                change.tag = event.tag
-                change.branch = None
-            elif event.ref and event.ref.startswith('refs/heads/'):
-                change = Branch(project)
-                change.branch = event.branch
-            else:
-                change = Ref(project)
-                change.branch = None
-            change.ref = event.ref
-            change.oldrev = event.oldrev
-            change.newrev = event.newrev
-            change.url = self.getGitwebUrl(project, sha=event.newrev)
-
-            # Pagure does not send files details in the git-receive event.
-            # Explicitly set files to None and let the pipelines processor
-            # call the merger asynchronuously
-            change.files = None
-
+                change_key.project_name, change_key.stable_id))
+            change = self._getNonPRRef(change_key, event=event)
         return change
 
-    def _getChange(self, project, number, patchset=None,
-                   refresh=False, url=None, event=None):
-        key = (project.name, number, patchset)
-        change = self._change_cache.get(key)
+    def _getChange(self, change_key, refresh=False, event=None):
+        log = get_annotated_logger(self.log, event)
+        number = int(change_key.stable_id)
+        change = self._change_cache.get(change_key)
         if change and not refresh:
-            self.log.debug("Getting change from cache %s" % str(key))
+            log.debug("Getting change from cache %s" % str(change_key))
             return change
+        project = self.source.getProject(change_key.project_name)
         if not change:
+            if not event:
+                self.log.error("Change %s not found in cache and no event",
+                               change_key)
+            if event:
+                url = event.change_url
             change = PullRequest(project.name)
             change.project = project
             change.number = number
             # patchset is the tips commit of the PR
-            change.patchset = patchset
-            change.url = url
+            change.patchset = change_key.revision
+            change.url = url or self.getPullUrl(project.name, number)
             change.uris = [
-                '%s/%s/pull/%s' % (self.baseurl, project, number),
+                '%s/%s/pull/%s' % (self.baseurl, project.name, number),
             ]
-        self._change_cache[key] = change
+
+        log.debug("Getting change pr#%s from project %s" % (
+            number, project.name))
+        log.info("Updating change from pagure %s" % change)
+        pull = self.getPull(change.project.name, change.number)
+
+        def _update_change(c):
+            self._updateChange(c, event, pull)
+
+        change = self._change_cache.updateChangeWithRetry(change_key, change,
+                                                          _update_change)
+        return change
+
+    def _getNonPRRef(self, change_key, refresh=False, event=None):
+        change = self._change_cache.get(change_key)
+        if change:
+            if refresh:
+                self._change_cache.updateChangeWithRetry(
+                    change_key, change, lambda: None)
+            return change
+        if not event:
+            self.log.error("Change %s not found in cache and no event",
+                           change_key)
+        project = self.source.getProject(change_key.project_name)
+        if change_key.change_type == 'Tag':
+            change = Tag(project)
+            tag = change_key.stable_id
+            change.tag = tag
+            change.ref = f'refs/tags/{tag}'
+        elif change_key.change_type == 'Branch':
+            branch = change_key.stable_id
+            change = Branch(project)
+            change.branch = branch
+            change.ref = f'refs/heads/{branch}'
+        else:
+            change = Ref(project)
+            change.ref = change_key.stable_id
+        change.oldrev = change_key.oldrev
+        change.newrev = change_key.newrev
+        change.url = self.getGitwebUrl(project, sha=change.newrev)
+        # Pagure does not send files details in the git-receive event.
+        # Explicitly set files to None and let the pipelines processor
+        # call the merger asynchronuously
+        change.files = None
         try:
-            self.log.debug("Getting change pr#%s from project %s" % (
-                number, project.name))
-            self._updateChange(change, event)
-        except Exception:
-            if key in self._change_cache:
-                del self._change_cache[key]
-            raise
+            self._change_cache.set(change_key, change)
+        except ConcurrentUpdateError:
+            change = self._change_cache.get(change_key)
         return change
 
     def _hasRequiredStatusChecks(self, change):
@@ -717,12 +762,12 @@ class PagureConnection(BaseConnection):
                     score_board[author] -= 1
         return sum(score_board.values())
 
-    def _updateChange(self, change, event):
-        self.log.info("Updating change from pagure %s" % change)
-        change.pr = self.getPull(change.project.name, change.number)
+    def _updateChange(self, change, event, pull):
+        change.pr = pull
         change.ref = "refs/pull/%s/head" % change.number
         change.branch = change.pr.get('branch')
-        change.patchset = change.pr.get('commit_stop')
+        change.is_current_patchset = (change.pr.get('commit_stop') ==
+                                      change.patchset)
         change.files = change.pr.get('files')
         change.title = change.pr.get('title')
         change.tags = change.pr.get('tags')
@@ -734,10 +779,6 @@ class PagureConnection(BaseConnection):
         # last_updated seems to be touch for comment changed/flags - that's OK
         change.updated_at = change.pr.get('last_updated')
         self.log.info("Updated change from pagure %s" % change)
-
-        if self.sched:
-            self.sched.onChangeUpdated(change, event)
-
         return change
 
     def commentPull(self, project, number, message):
@@ -769,11 +810,11 @@ class PagureConnection(BaseConnection):
         # a the depends-on string in PR initial message. Not a blocker
         # for now, let's workaround using the local change cache !
         changes_dependencies = []
-        for cached_change_id, _change in self._change_cache.items():
+        for cached_change in self._change_cache:
             for dep_header in dependson.find_dependency_headers(
-                    _change.message):
+                    cached_change.message):
                 if change.url in dep_header:
-                    changes_dependencies.append(_change)
+                    changes_dependencies.append(cached_change)
         return changes_dependencies
 
     def mergePull(self, project, number):
