@@ -27,7 +27,7 @@ from zuul.model import (
 )
 from zuul.zk.event_queues import PipelineResultEventQueue
 from zuul.zk.executor import ExecutorApi
-from zuul.zk.exceptions import BuildRequestNotFound
+from zuul.zk.exceptions import JobRequestNotFound
 from kazoo.exceptions import BadVersionError
 
 
@@ -38,8 +38,6 @@ class ExecutorClient(object):
     def __init__(self, config, sched):
         self.config = config
         self.sched = sched
-        self.builds = {}
-        self.meta_jobs = {}  # A list of meta-jobs like stop or describe
 
         self.executor_api = self._executor_api_class(self.sched.zk_client)
         self.result_events = PipelineResultEventQueue.createRegistry(
@@ -49,45 +47,47 @@ class ExecutorClient(object):
     def stop(self):
         self.log.debug("Stopping")
 
-    def execute(self, job, item, pipeline, dependent_changes=[],
-                merger_items=[]):
+    def execute(self, job, nodes, item, pipeline, executor_zone,
+                dependent_changes=[], merger_items=[]):
         log = get_annotated_logger(self.log, item.event)
         uuid = str(uuid4().hex)
-        nodeset = item.current_build_set.getJobNodeSet(job.name)
         log.info(
             "Execute job %s (uuid: %s) on nodes %s for change %s "
             "with dependent changes %s",
-            job, uuid, nodeset, item.change, dependent_changes)
+            job, uuid, nodes, item.change, dependent_changes)
 
         params = zuul.executor.common.construct_build_params(
-            uuid, self.sched, nodeset,
+            uuid, self.sched.connections,
             job, item, pipeline, dependent_changes, merger_items,
             redact_secrets_and_keys=False)
         # TODO: deprecate and remove this variable?
         params["zuul"]["_inheritance_path"] = list(job.inheritance_path)
 
-        build = Build(
-            job,
-            item.current_build_set,
-            uuid,
+        build = Build.new(
+            pipeline.manager.current_context,
+            job=job,
+            build_set=item.current_build_set,
+            uuid=uuid,
             zuul_event_id=item.event.zuul_event_id,
         )
-        build.parameters = params
 
         log.debug("Adding build %s of job %s to item %s",
                   build, job, item)
         item.addBuild(build)
-        self.builds[uuid] = build
 
         if job.name == 'noop':
             data = {"start_time": time.time()}
-            started_event = BuildStartedEvent(build.uuid, data)
+            started_event = BuildStartedEvent(
+                build.uuid, build.build_set.uuid, job.name, None, data,
+                zuul_event_id=build.zuul_event_id)
             self.result_events[pipeline.tenant.name][pipeline.name].put(
                 started_event
             )
 
             result = {"result": "SUCCESS", "end_time": time.time()}
-            completed_event = BuildCompletedEvent(build.uuid, result)
+            completed_event = BuildCompletedEvent(
+                build.uuid, build.build_set.uuid, job.name, None, result,
+                zuul_event_id=build.zuul_event_id)
             self.result_events[pipeline.tenant.name][pipeline.name].put(
                 completed_event
             )
@@ -102,17 +102,9 @@ class ExecutorClient(object):
 
         # Store the NodeRequest ID in the job arguments, so we can look it up
         # on the executor side to lock the nodes.
-        node_request = build.build_set.getJobNodeRequest(job.name)
-        if node_request:
-            params["noderequest_id"] = node_request.id
-
-        # Because all nodes belong to the same provider, region and
-        # availability zone we can get executor_zone from only the first
-        # node.
-        executor_zone = None
-        if params["nodes"] and params["nodes"][0].get('attributes'):
-            executor_zone = params[
-                "nodes"][0]['attributes'].get('executor-zone')
+        req_id = build.build_set.getJobNodeRequestID(job.name)
+        if req_id:
+            params["noderequest_id"] = req_id
 
         zone_known = False
         if executor_zone:
@@ -131,15 +123,19 @@ class ExecutorClient(object):
                 # Fall back to the default zone
                 executor_zone = None
 
-        build.build_request_ref = self.executor_api.submit(
+        request = BuildRequest(
             uuid=uuid,
+            build_set_uuid=build.build_set.uuid,
+            job_name=job.name,
             tenant_name=build.build_set.item.pipeline.tenant.name,
             pipeline_name=build.build_set.item.pipeline.name,
-            params=params,
             zone=executor_zone,
             event_id=item.event.zuul_event_id,
-            precedence=PRIORITY_MAP[pipeline.precedence],
+            precedence=PRIORITY_MAP[pipeline.precedence]
         )
+        self.executor_api.submit(request, params)
+        build.updateAttributes(pipeline.manager.current_context,
+                               build_request_ref=request.path)
 
     def cancel(self, build):
         log = get_annotated_logger(self.log, build.zuul_event_id,
@@ -147,7 +143,9 @@ class ExecutorClient(object):
         # Returns whether a running build was canceled
         log.info("Cancel build %s for job %s", build, build.job)
 
-        build.canceled = True
+        build.updateAttributes(
+            build.build_set.item.pipeline.manager.current_context,
+            canceled=True)
 
         if not build.build_request_ref:
             log.debug("Build has not been submitted to ZooKeeper")
@@ -177,7 +175,9 @@ class ExecutorClient(object):
                     result = {"result": "CANCELED", "end_time": time.time()}
                     tenant_name = build.build_set.item.pipeline.tenant.name
                     pipeline_name = build.build_set.item.pipeline.name
-                    event = BuildCompletedEvent(build_request.uuid, result)
+                    event = BuildCompletedEvent(
+                        build_request.uuid, build_request.build_set_uuid,
+                        build_request.job_name, build_request.path, result)
                     self.result_events[tenant_name][pipeline_name].put(event)
                 finally:
                     self.executor_api.unlock(build_request)
@@ -195,7 +195,7 @@ class ExecutorClient(object):
 
         return False
 
-    def resumeBuild(self, build: Build) -> bool:
+    def resumeBuild(self, build):
         log = get_annotated_logger(self.log, build.zuul_event_id)
 
         if not build.build_request_ref:
@@ -209,22 +209,25 @@ class ExecutorClient(object):
             return True
         return False
 
-    def removeBuild(self, build: Build) -> None:
+    def removeBuild(self, build):
         log = get_annotated_logger(self.log, build.zuul_event_id)
         log.debug("Removing build %s", build.uuid)
 
         if not build.build_request_ref:
-            log.debug("Build has not been submitted to ZooKeeper")
+            log.debug("Build %s has not been submitted to ZooKeeper",
+                      build.uuid)
             return
 
         build_request = self.executor_api.get(build.build_request_ref)
         if build_request:
+            # TODO (felix): We could directly remove the build request via
+            # its path in ZK to spare a read operation. Usually there should
+            # be no need to look up the build request object from ZooKeeper
+            # just to immediately remove it.
             self.executor_api.remove(build_request)
 
-        del self.builds[build.uuid]
-
     def cleanupLostBuildRequests(self):
-        for build_request in self.executor_api.lostBuildRequests():
+        for build_request in self.executor_api.lostRequests():
             try:
                 self.cleanupLostBuildRequest(build_request)
             except Exception:
@@ -241,8 +244,11 @@ class ExecutorClient(object):
         build_request.state = BuildRequest.COMPLETED
         try:
             self.executor_api.update(build_request)
-        except BuildRequestNotFound as e:
+        except JobRequestNotFound as e:
             self.log.warning("Could not complete build: %s", str(e))
+            # In case we re-created the lock directory, still remove
+            # the request for the side effect of removing the lock.
+            self.executor_api.remove(build_request)
             return
         except BadVersionError:
             # There could be a race condition:
@@ -255,6 +261,8 @@ class ExecutorClient(object):
 
         # No need to unlock the build, as it is by definition unlocked
 
-        event = BuildCompletedEvent(build_request.uuid, result)
+        event = BuildCompletedEvent(
+            build_request.uuid, build_request.build_set_uuid,
+            build_request.job_name, build_request.path, result)
         self.result_events[build_request.tenant_name][
             build_request.pipeline_name].put(event)
