@@ -1,6 +1,7 @@
 # Copyright 2014 OpenStack Foundation
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 # Copyright 2016 Red Hat
+# Copyright 2022 Acme Gating, LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,6 +15,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
 import logging
 import os
 import socket
@@ -22,6 +24,49 @@ import threading
 from zuul.lib.queue import NamedQueue
 
 
+class Command:
+    name = None
+    help = None
+    args = []
+
+
+class Argument:
+    name = None
+    help = None
+    required = None
+    default = None
+
+
+class StopCommand(Command):
+    name = 'stop'
+    help = 'Stop the running process'
+
+
+class GracefulCommand(Command):
+    name = 'graceful'
+    help = 'Stop after completing existing work'
+
+
+class PauseCommand(Command):
+    name = 'pause'
+    help = 'Stop accepting new work'
+
+
+class UnPauseCommand(Command):
+    name = 'unpause'
+    help = 'Resume accepting new work'
+
+
+class ReplCommand(Command):
+    name = 'repl'
+    help = 'Enable the REPL for debugging'
+
+
+class NoReplCommand(Command):
+    name = 'norepl'
+    help = 'Disable the REPL'
+
+
 class CommandSocket(object):
     log = logging.getLogger("zuul.CommandSocket")
 
@@ -54,7 +99,7 @@ class CommandSocket(object):
         # either handle '_stop' or just ignore the unknown command and
         # then check to see if they should continue to run before
         # re-entering their loop.
-        self.queue.put(b'_stop')
+        self.queue.put(('_stop', []))
         self.socket_thread.join()
 
     def _socketListener(self):
@@ -70,11 +115,17 @@ class CommandSocket(object):
                 buf = buf.strip()
                 self.log.debug("Received %s from socket" % (buf,))
                 s.close()
+
+                buf = buf.decode('utf8')
+                parts = buf.split(' ', 1)
                 # Because we use '_stop' internally to wake up a
                 # waiting thread, don't allow it to actually be
                 # injected externally.
-                if buf != b'_stop':
-                    self.queue.put(buf)
+                args = parts[1:]
+                if args:
+                    args = json.loads(args[0])
+                if parts[0] != '_stop':
+                    self.queue.put((parts[0], args))
             except Exception:
                 self.log.exception("Exception in socket handler")
 
