@@ -13,7 +13,6 @@
 import json
 import types
 
-import zuul.configloader
 import zuul.model
 
 
@@ -27,10 +26,10 @@ class ZuulJSONEncoder(json.JSONEncoder):
             return d
         elif (
                 isinstance(o, zuul.model.SourceContext) or
-                isinstance(o, zuul.configloader.ZuulMark)):
+                isinstance(o, zuul.model.ZuulMark)):
             return {}
         return json.JSONEncoder.default(self, o)
 
 
-def json_dumps(obj):
-    return json.dumps(obj, cls=ZuulJSONEncoder)
+def json_dumps(obj, **kw):
+    return json.dumps(obj, cls=ZuulJSONEncoder, **kw)
