@@ -91,7 +91,8 @@ class PipelineManager(metaclass=ABCMeta):
         finally:
             self.current_context = None
 
-    def _postConfig(self, layout):
+    def _postConfig(self):
+        layout = self.pipeline.tenant.layout
         # If our layout UUID already matches the UUID in ZK, we don't
         # need to make any changes in ZK.  But we do still need to
         # update our local object pointers.  Note that our local queue
@@ -227,6 +228,8 @@ class PipelineManager(metaclass=ABCMeta):
                 change = source.getChange(key)
                 if change is None:
                     self.log.error("Unable to resolve change from key %s", key)
+                if isinstance(change, model.Change):
+                    self.updateCommitDependencies(change, None, event=None)
                 self._change_cache[change.cache_key] = change
             resolved_changes.append(change)
         return resolved_changes
@@ -284,6 +287,8 @@ class PipelineManager(metaclass=ABCMeta):
 
         change_in_pipeline = False
         for item in self.pipeline.getAllItems():
+            if not isinstance(change, model.Change):
+                continue
             for dep_change_ref in item.change.commit_needs_changes:
                 if item.change.equals(change):
                     change_in_pipeline = True
@@ -312,6 +317,13 @@ class PipelineManager(metaclass=ABCMeta):
                 self.log.error("Reporting item start %s received: %s" %
                                (item, ret))
 
+    def reportNormalBuildsetEnd(self, build_set, action, final, result=None):
+        # Report a buildset end, but only if there are jobs
+        if (build_set.job_graph and
+            len(build_set.job_graph.jobs) > 0):
+            self.sql.reportBuildsetEnd(build_set, action,
+                                       final, result)
+
     def reportDequeue(self, item):
         if not self.pipeline.state.disabled:
             self.log.info(
@@ -326,8 +338,8 @@ class PipelineManager(metaclass=ABCMeta):
                 )
         # This might be called after canceljobs, which also sets a
         # non-final 'cancel' result.
-        self.sql.reportBuildsetEnd(item.current_build_set, 'dequeue',
-                                   final=False)
+        self.reportNormalBuildsetEnd(item.current_build_set, 'dequeue',
+                                     final=False)
 
     def sendReport(self, action_reporters, item, message=None):
         """Sends the built message off to configured reporters.
@@ -610,6 +622,8 @@ class PipelineManager(metaclass=ABCMeta):
         if self.pipeline.tenant.layout.getProjectPipelineConfig(ci):
             self.sendReport(actions, ci)
         self.dequeueItem(ci)
+        # We don't use reportNormalBuildsetEnd here because we want to
+        # report even with no jobs.
         self.sql.reportBuildsetEnd(ci.current_build_set,
                                    'failure', final=True)
 
@@ -946,7 +960,7 @@ class PipelineManager(metaclass=ABCMeta):
                 item.didBundleStartReporting()):
             # Force a dequeued result here because we haven't actually
             # reported the item, but we are done with this buildset.
-            self.sql.reportBuildsetEnd(
+            self.reportNormalBuildsetEnd(
                 item.current_build_set, 'dequeue', final=False,
                 result='DEQUEUED')
             item.resetAllBuilds()
@@ -1245,7 +1259,10 @@ class PipelineManager(metaclass=ABCMeta):
         log.debug("Scheduling fileschanged for item %s", item)
         build_set = item.current_build_set
 
-        to_sha = getattr(item.change, "branch", None)
+        # if base_sha is not available, fallback to branch
+        to_sha = getattr(item.change, "base_sha",
+                         getattr(item.change, "branch", None))
+
         self.sched.merger.getFilesChanges(
             item.change.project.connection_name, item.change.project.name,
             item.change.ref, to_sha, build_set=build_set,
@@ -1583,9 +1600,9 @@ class PipelineManager(metaclass=ABCMeta):
                     request_id, cached=True)
                 if not node_request:
                     continue
-                if node_request.fulfilled:
-                    # If the node request is already fulfilled, there is no
-                    # need to update the relative priority.
+                if node_request.state != model.STATE_REQUESTED:
+                    # If the node request was locked and accepted by a
+                    # provider, we can no longer update the relative priority.
                     continue
                 if node_request.relative_priority != priority:
                     self.sched.nodepool.reviseRequest(
@@ -1607,8 +1624,8 @@ class PipelineManager(metaclass=ABCMeta):
                 # Don't override the reported sql result for the item
                 # that "really" failed.
                 ri.setReportedResult('FAILURE')
-                self.sql.reportBuildsetEnd(ri.current_build_set,
-                                           'failure', final=True)
+                self.reportNormalBuildsetEnd(ri.current_build_set,
+                                             'failure', final=True)
 
     def processQueue(self):
         # Do whatever needs to be done for each change in the queue
@@ -1865,8 +1882,8 @@ class PipelineManager(metaclass=ABCMeta):
                               item.change)
                     action = 'merge-failure'
                     item.setReportedResult('MERGE_FAILURE')
-                self.sql.reportBuildsetEnd(item.current_build_set,
-                                           action, final=True)
+                self.reportNormalBuildsetEnd(item.current_build_set,
+                                             action, final=True)
             change_queue = item.queue
             if not (succeeded and merged):
                 if (not item.current_build_set.job_graph or
@@ -1886,8 +1903,8 @@ class PipelineManager(metaclass=ABCMeta):
                 raise exceptions.MergeFailure(
                     "Change %s failed to merge" % item.change)
             else:
-                self.sql.reportBuildsetEnd(item.current_build_set,
-                                           action, final=True)
+                self.reportNormalBuildsetEnd(item.current_build_set,
+                                             action, final=True)
                 log.info("Reported change %s status: all-succeeded: %s, "
                          "merged: %s", item.change, succeeded, merged)
                 change_queue.increaseWindowSize()
@@ -1898,8 +1915,8 @@ class PipelineManager(metaclass=ABCMeta):
                 tenant = self.pipeline.tenant
                 zuul_driver.onChangeMerged(tenant, item.change, source)
         elif action:
-            self.sql.reportBuildsetEnd(item.current_build_set,
-                                       action, final=True)
+            self.reportNormalBuildsetEnd(item.current_build_set,
+                                         action, final=True)
 
     def _reportItem(self, item):
         log = get_annotated_logger(self.log, item.event)
