@@ -401,13 +401,18 @@ class Repo(object):
         log.debug("Resetting repository %s", self.local_path)
         self.createRepoObject(zuul_event_id, build=build)
 
-        if process_worker is None:
-            self._reset(self.local_path, self.env, log)
-        else:
-            job = process_worker.submit(Repo._reset, self.local_path, self.env)
-            messages = job.result()
-            for message in messages:
-                log.debug(message)
+        try:
+            if process_worker is None:
+                self._reset(self.local_path, self.env, log)
+            else:
+                job = process_worker.submit(
+                    Repo._reset, self.local_path, self.env)
+                messages = job.result()
+                for message in messages:
+                    log.debug(message)
+        except Exception:
+            shutil.rmtree(self.local_path)
+            raise
 
     def getBranchHead(self, branch, zuul_event_id=None):
         repo = self.createRepoObject(zuul_event_id)
@@ -509,12 +514,11 @@ class Repo(object):
         log = get_annotated_logger(self.log, zuul_event_id)
         repo = self.createRepoObject(zuul_event_id)
         try:
-            origin_ref = repo.remotes.origin.refs[branch]
+            log.debug("Updating remote reference origin/%s to %s", branch, rev)
+            repo.remotes.origin.refs[branch].commit = rev
         except IndexError:
-            log.warning("No remote ref found for branch %s", branch)
-            return
-        log.debug("Updating remote reference %s to %s", origin_ref, rev)
-        origin_ref.commit = rev
+            log.warning("No remote ref found for branch %s, creating", branch)
+            Repo._setRef(f"refs/remotes/origin/{branch}", str(rev), repo)
 
     def deleteRef(self, path, repo=None, zuul_event_id=None):
         ref_log = get_annotated_logger(
@@ -690,9 +694,22 @@ class Repo(object):
         files = set()
 
         if tosha:
+            # When "tosha" is the target branch, the result of diff() correctly
+            # excluds the files whose changes are reverted between the commits.
+            # But it may also include the files that are not changed in the
+            # referenced commit(s). This can happen, e.g. if the target branch
+            # has diverged from the feature branch.
+            # The idea is to use this result to filter out the files whose
+            # changes are reverted between the commits.
+            diff_files = set()
+            head_commit = repo.commit(head.hexsha)
+            diff_index = head_commit.diff(tosha)
+            diff_files.update((item.a_path for item in diff_index))
+
             commit_diff = "{}..{}".format(tosha, head.hexsha)
             for cmt in repo.iter_commits(commit_diff, no_merges=True):
-                files.update(cmt.stats.files.keys())
+                files.update(f for f in cmt.stats.files.keys()
+                             if f in diff_files)
         else:
             files.update(head.stats.files.keys())
         return list(files)
