@@ -65,7 +65,8 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             mark = err.key.mark
             if not (context and mark and err.short_error):
                 continue
-            if context.project != item.change.project:
+            if context.project_canonical_name != \
+                    item.change.project.canonical_name:
                 continue
             if not hasattr(item.change, 'branch'):
                 continue
@@ -121,6 +122,7 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             'start': self._formatItemReportStart,
             'success': self._formatItemReportSuccess,
             'failure': self._formatItemReportFailure,
+            'merge-conflict': self._formatItemReportMergeConflict,
             'merge-failure': self._formatItemReportMergeFailure,
             'no-jobs': self._formatItemReportNoJobs,
             'disabled': self._formatItemReportDisabled,
@@ -148,8 +150,7 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
         return ret
 
     def _formatItemReportEnqueue(self, item, with_jobs=True):
-        status_url = get_default(self.connection.sched.config,
-                                 'web', 'status_url', '')
+        status_url = self.connection.sched.globals.web_status_url
         if status_url:
             status_url = item.formatUrlPattern(status_url)
 
@@ -159,8 +160,7 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             status_url=status_url)
 
     def _formatItemReportStart(self, item, with_jobs=True):
-        status_url = get_default(self.connection.sched.config,
-                                 'web', 'status_url', '')
+        status_url = self.connection.sched.globals.web_status_url
         if status_url:
             status_url = item.formatUrlPattern(status_url)
 
@@ -180,6 +180,9 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             msg = 'This change is part of a bundle that failed to merge.\n'
         elif item.dequeued_needing_change:
             msg = 'This change depends on a change that failed to merge.\n'
+        elif item.dequeued_missing_requirements:
+            msg = ('This change is unable to merge '
+                   'due to a missing requirement.\n')
         elif item.isBundleFailing():
             msg = 'This change is part of a bundle that failed.\n'
             if with_jobs:
@@ -187,7 +190,7 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             msg = "{}\n\n{}".format(
                 msg, self._formatItemReportOtherBundleItems(item))
         elif item.didMergerFail():
-            msg = item.pipeline.merge_failure_message
+            msg = item.pipeline.merge_conflict_message
         elif item.getConfigErrors():
             msg = str(item.getConfigErrors()[0].error)
         else:
@@ -196,8 +199,11 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
                 msg += '\n\n' + self._formatItemReportJobs(item)
         return msg
 
+    def _formatItemReportMergeConflict(self, item, with_jobs=True):
+        return item.pipeline.merge_conflict_message
+
     def _formatItemReportMergeFailure(self, item, with_jobs=True):
-        return item.pipeline.merge_failure_message
+        return 'This change was not merged by the code review system.\n'
 
     def _formatItemReportNoJobs(self, item, with_jobs=True):
         status_url = get_default(self.connection.sched.config,
@@ -225,8 +231,10 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
         return msg
 
     def _formatItemReportOtherBundleItems(self, item):
+        related_changes = item.pipeline.manager.resolveChangeReferences(
+            item.change.needs_changes)
         return "Related changes:\n{}".format("\n".join(
-            c.url for c in item.change.needs_changes if c is not item.change))
+            c.url for c in related_changes if c is not item.change))
 
     def _getItemReportJobsFields(self, item):
         # Extract the report elements from an item
@@ -270,7 +278,11 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             # the url field to match old behavior until we can deal with
             # the gerrit-side piece as well
             url = url or job.name
-            jobs_fields.append((name, url, result, error, elapsed, voting))
+            # Pass user defined success_message deciding the build result
+            # during result formatting
+            success_message = job.success_message
+            jobs_fields.append(
+                (name, url, result, error, elapsed, voting, success_message))
         return jobs_fields
 
     def _formatItemReportJobs(self, item):
@@ -278,5 +290,5 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
         ret = ''
         jobs_fields = self._getItemReportJobsFields(item)
         for job_fields in jobs_fields:
-            ret += '- %s%s : %s%s%s%s\n' % job_fields
+            ret += '- %s%s : %s%s%s%s\n' % job_fields[:6]
         return ret
