@@ -21,7 +21,9 @@ from urllib.parse import quote_plus, unquote_plus
 
 from kazoo.exceptions import NoNodeError
 
+from zuul import model
 from zuul.zk import sharding, ZooKeeperSimpleBase
+from zuul.zk.vendor import lock
 
 CONFIG_ROOT = "/zuul/config"
 
@@ -51,7 +53,7 @@ class FilesCache(ZooKeeperSimpleBase, MutableMapping):
             "extra_dirs_searched": list(extra_config_dirs),
             "ltime": ltime,
         }
-        payload = json.dumps(data).encode("utf8")
+        payload = json.dumps(data, sort_keys=True).encode("utf8")
         try:
             self.kazoo_client.set(self.root_path, payload)
         except NoNodeError:
@@ -132,6 +134,10 @@ class FilesCache(ZooKeeperSimpleBase, MutableMapping):
             children = []
         return len(children)
 
+    def clear(self):
+        with contextlib.suppress(NoNodeError):
+            self.kazoo_client.delete(self.root_path, recursive=True)
+
 
 class UnparsedConfigCache(ZooKeeperSimpleBase):
     """Zookeeper cache for unparsed config files."""
@@ -144,12 +150,12 @@ class UnparsedConfigCache(ZooKeeperSimpleBase):
         self.lock_path = f"{config_root}/lock"
 
     def readLock(self, project_cname):
-        return self.kazoo_client.ReadLock(
-            _safe_path(self.lock_path, project_cname))
+        return lock.ReadLock(
+            self.kazoo_client, _safe_path(self.lock_path, project_cname))
 
     def writeLock(self, project_cname):
-        return self.kazoo_client.WriteLock(
-            _safe_path(self.lock_path, project_cname))
+        return lock.WriteLock(
+            self.kazoo_client, _safe_path(self.lock_path, project_cname))
 
     def getFilesCache(self, project_cname, branch_name):
         path = _safe_path(self.cache_path, project_cname, branch_name)
@@ -169,3 +175,67 @@ class UnparsedConfigCache(ZooKeeperSimpleBase):
             path = _safe_path(self.cache_path, project_cname, branch_name)
         with contextlib.suppress(NoNodeError):
             self.kazoo_client.delete(path, recursive=True)
+
+
+class SystemConfigCache(ZooKeeperSimpleBase):
+    """Zookeeper cache for Zuul system configuration.
+
+    The system configuration consists of the unparsed abide config and
+    the runtime related settings from the Zuul config file.
+    """
+
+    SYSTEM_ROOT = "/zuul/system"
+    log = logging.getLogger("zuul.zk.SystemConfigCache")
+
+    def __init__(self, client, callback):
+        super().__init__(client)
+        self.conf_path = f"{self.SYSTEM_ROOT}/conf"
+        self.lock_path = f"{self.SYSTEM_ROOT}/conf-lock"
+        self._callback = callback
+        self.kazoo_client.DataWatch(self.conf_path, self._configChanged)
+
+    def _configChanged(self, data, stat, event):
+        self._callback()
+
+    @property
+    def ltime(self):
+        with lock.ReadLock(self.kazoo_client, self.lock_path):
+            zstat = self.kazoo_client.exists(self.conf_path)
+        return -1 if zstat is None else zstat.last_modified_transaction_id
+
+    @property
+    def is_valid(self):
+        return self.ltime > -1
+
+    def get(self):
+        """Get the system configuration from Zookeeper.
+
+        :returns: A tuple (unparsed abide config, system attributes)
+        """
+        with lock.ReadLock(self.kazoo_client, self.lock_path):
+            try:
+                with sharding.BufferedShardReader(
+                    self.kazoo_client, self.conf_path
+                ) as stream:
+                    data = json.loads(stream.read())
+            except Exception:
+                raise RuntimeError("No valid system config")
+            zstat = self.kazoo_client.exists(self.conf_path)
+            return (model.UnparsedAbideConfig.fromDict(
+                    data["unparsed_abide"],
+                    ltime=zstat.last_modified_transaction_id
+                ), model.SystemAttributes.fromDict(data["system_attributes"]))
+
+    def set(self, unparsed_abide, system_attributes):
+        with lock.WriteLock(self.kazoo_client, self.lock_path):
+            data = {
+                "unparsed_abide": unparsed_abide.toDict(),
+                "system_attributes": system_attributes.toDict(),
+            }
+            with sharding.BufferedShardWriter(
+                self.kazoo_client, self.conf_path
+            ) as stream:
+                stream.truncate(0)
+                stream.write(json.dumps(data, sort_keys=True).encode("utf8"))
+            zstat = self.kazoo_client.exists(self.conf_path)
+            unparsed_abide.ltime = zstat.last_modified_transaction_id
