@@ -14,10 +14,10 @@
 
 import io
 from contextlib import suppress
+import time
 import zlib
 
 from kazoo.exceptions import NoNodeError
-from kazoo.client import TransactionRequest
 
 # The default size limit for a node in Zookeeper is ~1MiB. However, as this
 # also includes the size of the key we can not use all of it for data.
@@ -29,7 +29,12 @@ class RawShardIO(io.RawIOBase):
     def __init__(self, client, path):
         self.client = client
         self.shard_base = path
-        self.is_transaction = isinstance(client, TransactionRequest)
+        self.compressed_bytes_read = 0
+        self.compressed_bytes_written = 0
+        self.cumulative_read_time = 0.0
+        self.cumulative_write_time = 0.0
+        self.znodes_read = 0
+        self.znodes_written = 0
 
     def readable(self):
         return True
@@ -40,20 +45,25 @@ class RawShardIO(io.RawIOBase):
     def truncate(self, size=None):
         if size != 0:
             raise ValueError("Can only truncate to 0")
-        if self.is_transaction:
-            raise Exception("Truncate within transactions unimplemented")
         with suppress(NoNodeError):
             self.client.delete(self.shard_base, recursive=True)
 
     @property
     def _shards(self):
         try:
-            return self.client.get_children(self.shard_base)
+            start = time.perf_counter()
+            ret = self.client.get_children(self.shard_base)
+            self.cumulative_read_time += time.perf_counter() - start
+            return ret
         except NoNodeError:
             return []
 
     def _getData(self, path):
+        start = time.perf_counter()
         data, _ = self.client.get(path)
+        self.cumulative_read_time += time.perf_counter() - start
+        self.compressed_bytes_read += len(data)
+        self.znodes_read += 1
         return zlib.decompress(data)
 
     def readall(self):
@@ -68,23 +78,52 @@ class RawShardIO(io.RawIOBase):
         # Only write one key at a time and defer writing the rest to the caller
         shard_bytes = bytes(shard_data[0:NODE_BYTE_SIZE_LIMIT])
         shard_bytes = zlib.compress(shard_bytes)
-        assert(len(shard_bytes) < NODE_BYTE_SIZE_LIMIT)
-        kw = dict(sequence=True)
-        if not self.is_transaction:
-            kw['makepath'] = True
+        if not (len(shard_bytes) < NODE_BYTE_SIZE_LIMIT):
+            raise RuntimeError("Shard too large")
+        start = time.perf_counter()
         self.client.create(
             "{}/".format(self.shard_base),
             shard_bytes,
-            **kw
+            sequence=True,
+            makepath=True,
         )
+        self.cumulative_write_time += time.perf_counter() - start
+        self.compressed_bytes_written += len(shard_bytes)
+        self.znodes_written += 1
         return min(byte_count, NODE_BYTE_SIZE_LIMIT)
 
 
 class BufferedShardWriter(io.BufferedWriter):
     def __init__(self, client, path):
-        super().__init__(RawShardIO(client, path), NODE_BYTE_SIZE_LIMIT)
+        self.__raw = RawShardIO(client, path)
+        super().__init__(self.__raw, NODE_BYTE_SIZE_LIMIT)
+
+    @property
+    def compressed_bytes_written(self):
+        return self.__raw.compressed_bytes_written
+
+    @property
+    def cumulative_write_time(self):
+        return self.__raw.cumulative_write_time
+
+    @property
+    def znodes_written(self):
+        return self.__raw.znodes_written
 
 
 class BufferedShardReader(io.BufferedReader):
     def __init__(self, client, path):
-        super().__init__(RawShardIO(client, path), NODE_BYTE_SIZE_LIMIT)
+        self.__raw = RawShardIO(client, path)
+        super().__init__(self.__raw, NODE_BYTE_SIZE_LIMIT)
+
+    @property
+    def compressed_bytes_read(self):
+        return self.__raw.compressed_bytes_read
+
+    @property
+    def cumulative_read_time(self):
+        return self.__raw.cumulative_read_time
+
+    @property
+    def znodes_read(self):
+        return self.__raw.znodes_read
